from __future__ import annotations

import importlib
import inspect
import pkgutil
from types import ModuleType

from docutils import nodes
from docutils.parsers.rst import directives
from docutils.statemachine import StringList
from sphinx.application import Sphinx
from sphinx.util.docutils import SphinxDirective


class AutoMembers(SphinxDirective):
    required_arguments = 1
    option_spec = {
        "recursive": directives.flag,
        "exclude": directives.unchanged,
    }
    _excluded_members: list[str] = None

    @property
    def module_name(self) -> str:
        return self.arguments[0]

    @property
    def excluded_members(self) -> list[str]:
        if self._excluded_members is None:
            self._excluded_members = [member.strip() for member in self.options.get("exclude", "").split(",")]
        return self._excluded_members

    def run(self):
        return [node for member in self.get_members(self.module_name).values() for node in self.nodes_from_rst(member)]

    def nodes_from_rst(self, rst: list[str]) -> list[nodes.Node]:
        wrapper = nodes.paragraph()
        self.state.nested_parse(StringList(rst), 0, wrapper)
        return wrapper.children

    def should_include(self, obj: object) -> bool:
        module_or_name = getattr(obj, "__module__", obj.__name__)
        return (self.module_name == module_or_name or f"{self.module_name}." in module_or_name and
                obj.__name__ not in self.excluded_members)

    def get_members(self, module: str | ModuleType) -> dict[str, list[str]]:
        autodoc_options = self.env.config.automembers_autodoc_options
        members = {}
        try:
            # If the module is already imported, use it directly
            module = importlib.import_module(module)
            # Class members
            classes = [
                member
                for member in inspect.getmembers(module, inspect.isclass)
                if self.should_include(member[1])
            ]
            class_members = {
                cls.__qualname__:
                [f".. currentmodule:: {module.__name__}"]
                + [f".. autoclass:: {cls.__qualname__}"]
                + [f"    {option}" for option in autodoc_options]
                for name, cls in classes if cls.__qualname__ not in members
            }
            members.update(class_members)
            # Function members
            functions = [
                member
                for member in inspect.getmembers(module, inspect.isfunction)
                if self.should_include(member[1])
            ]
            function_members = {
                func.__qualname__:
                [f".. currentmodule:: {module.__name__}"]
                + [f".. autofunction:: {func.__qualname__}"]
                for name, func in functions if func.__qualname__ not in members
            }
            members.update(function_members)
            # submodule members
            if "recursive" in self.options and "__path__" in dir(module):
                for moduleinfo in pkgutil.iter_modules(module.__path__):
                    fullname = f"{module.__name__}.{moduleinfo.name}"
                    if self.module_name not in fullname or fullname == self.module_name:
                        continue
                    members.update(self.get_members(fullname))
            return members
        except ImportError:
            return members


def setup(app: Sphinx) -> dict:
    app.add_directive("automembers", AutoMembers)
    app.add_config_value("automembers_recursive", False, "env")
    app.add_config_value("automembers_autodoc_options", [], "env")

    return {
        "version": "0.1",
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }



"""
Open Odb and Display a Contour Plot
===================================

`viewerOpenOdbAndContour.py` from `Opening an output database and displaying a contour plot <https://help.3ds.com/2021/English/DSSIMULIA_Established/SIMACAECMDRefMap/simacmd-c-intexaopenodb.htm?contextscope=all&id=0d282d5c2d004c5d9f115b9e7853cacf>`_.

Print a contour plot to a local PNG-format file.

Run the following command before running this script:

.. code-block:: sh

    abaqus fetch job=viewer_tutorial

The following example of a script containing Abaqus Scripting Interface commands uses the output database used by `Viewing the Output from Your Analysis <https://help.3ds.com/2021/English/DSSIMULIA_Established/SIMACAEGSARefMap/simagsa-m-Caevismod-sb.htm?contextscope=all#simagsa-m-Caevismod-sb>`_.

Use the following command to retrieve the example script:

.. code-block:: sh

    abaqus fetch job=viewerOpenOdbAndContour

The script does the following:

- Creates a viewport, and makes it the current viewport.
- Opens an output database.
- Displays a contour plot.
- Displays the model in the first frame of the third step.
- Sets the number of contour intervals and the contour limits.
- Prints a color image of the viewport to a .png file.
"""

from abaqus import *
from abaqusConstants import *
import visualization

# Create a new Viewport for this example.

myViewport=session.Viewport(name='Print a contour plot', 
    origin=(10, 10), width=200, height=100)

# Open the output database and associate it
# with the new viewport.

odbPath = "viewer_tutorial.odb"
myOdb = visualization.openOdb(path=odbPath)
    
myViewport.setValues(displayedObject=myOdb)


# Display a contour plot of the output database.

myViewport.odbDisplay.display.setValues(plotState=(CONTOURS_ON_DEF,))

# Change to the first frame of the third step.
# Remember that indices in Python begin with zero: 
#   The index of the first frame is 0.
#   The index of the third step is 2.

myViewport.odbDisplay.setFrame(step=2, frame=0)

# Change the number of contour intervals to 10
# starting at 0.0 and ending at 0.10.

myViewport.odbDisplay.contourOptions.setValues(numIntervals=10, 
    maxAutoCompute=OFF, maxValue=0.10, 
    minAutoCompute=OFF, minValue=0.0,)

# Generate color output. 
# Do not print the viewport decorations or the black background.

session.printOptions.setValues(rendition=COLOR,
    vpDecorations=OFF, vpBackground=OFF)

# Print the viewport to a local PNG-format file.

session.printToFile(fileName='contourPlot', format=PNG,
    canvasObjects=(myViewport,))



from typing import Optional

from abqpy.decorators import abaqus_function_doc

from ..UtilityAndView.abaqusConstants import Boolean, OFF

"""The methodCallback module provides functions that allow you to register a callback that 
will be invoked when certain Abaqus Scripting Interface commands are about to be 
executed. This callback allows you to augment the standard behavior of Abaqus Scripting 
Interface commands. 
For example, in the following script the myCallback function is executed when the 
writeInput method of an object whose type is JobType (in other words, any Job object) is 
about to be called::

    import methodCallback 
    from job import JobType 
    
    def myCallback(callingObject, arguments, keywordArguments, userData): 
        print 'An input file  is about to be written.' 
    methodCallback.addCallback(JobType, 'writeInput', myCallback) 

.. note:: 
    This object can be accessed by::

        import methodCallback

"""


@abaqus_function_doc
def addCallback(
    caller: str,
    methodName: str,
    callback: str,
    userData: Optional[str] = None,
    callAfter: Boolean = OFF,
):
    """This method adds a callback function that will be invoked when certain Abaqus/CAE
    commands are about to be executed.

    .. note::
        This function can be accessed by::

            methodCallback.addCallback

    Parameters
    ----------
    caller
        An object or type object specifying which object will trigger the callback function to
        be invoked or the Symbolic Constant ALL_TYPES.
    methodName
        A String specifying the name of the method on the caller that will trigger the callback
        function to be invoked or the Symbolic Constant ALL_METHODS.
    callback
        A Python function to be called when a command matching the specified caller and method
        name is about to be executed. The interface definition of the callback function is `def
        functionName(callingMethod, args, keywordArgs, userData)` where **callingMethod** is the
        method that called this function. **args** is the sequence of non-keyword arguments that
        was passed to the calling method. **keywordArgs** is the dictionary of keyword arguments
        that was passed to the calling method. **userData** is the object passed as the **userData**
        argument to the addCallback method.
    userData
        Any type of data. This data will be passed to the callback function. The default value
        is None.
    callAfter
        A Boolean specifying that the callback should be called after the method has executed
        (instead of before the method is called). The default value is False, which indicates
        that the callback should be called before the method has executed. If **callAfter** = True,
        you can also access the return value of the command from within the callback by
        including the following statement `returnValue = getMethodReturnValue()` The
        getMethodReturnValue function is in the global namespace of the callback function.
    """
    ...


@abaqus_function_doc
def removeCallback(caller: str, methodName: str, callback: str, userData: Optional[str] = None):
    """This method removes a callback added by the addCallback method. To successfully remove a
    callback, all arguments must exactly match those used when the callback was added.

    .. note::
        This function can be accessed by::

            methodCallback.removeCallback

    Parameters
    ----------
    caller
        An object or type object specifying which object will trigger the callback function to
        be invoked or the Symbolic Constant ALL_TYPES.
    methodName
        A String specifying the name of the method on the caller that will trigger the callback
        function to be invoked or the Symbolic Constant ALL_METHODS.
    callback
        A Python function to be called when a command matching the specified caller and method
        name is about to be executed. The interface definition of the callback function is `def
        functionName(callingMethod, args, keywordArgs, userData)` where **callingMethod** is the
        method that called this function. **args** is the sequence of nonkeyword arguments that was
        passed to the calling method. **keywordArgs** is the dictionary of keyword arguments that
        was passed to the calling method. **userData** is the object passed as the **userData**
        argument to the removeCallback method.
    userData
        Any type of data. This data will be passed to the callback function. The default value
        is None.
    """
    ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .BrittleFailure import BrittleFailure
from .BrittleShear import BrittleShear
from ....UtilityAndView.abaqusConstants import Boolean, OFF, STRAIN
from ....UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class BrittleCracking:
    """The BrittleCracking object specifies cracking and postcracking properties for the
    brittle cracking material model.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].brittleCracking
            import odbMaterial
            session.odbs[name].materials[name].brittleCracking

        The table data for this object are:

        - If **type** = STRAIN the table data specify the following:

            - Remaining direct stress after cracking.
            - Direct cracking strain.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **type** = DISPLACEMENT the table data specify the following:

            - Remaining direct stress after cracking.
            - Direct cracking displacement.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **type** = GFI the table data specify the following:

            - Failure stress.
            - Mode I fracture energy.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        The corresponding analysis keywords are:

        - BRITTLE CRACKING
    """

    #: A BrittleShear object.
    brittleShear: BrittleShear = BrittleShear(((),))

    #: A BrittleFailure object.
    brittleFailure: BrittleFailure = BrittleFailure(((),))

    @abaqus_method_doc
    def __init__(
        self,
        table: tuple,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        type: Literal[C.STRAIN, C.GFI, C.DISPLACEMENT] = STRAIN,
    ):
        """This method creates a BrittleCracking object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].BrittleCracking
                session.odbs[name].materials[name].BrittleCracking

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        type
            A SymbolicConstant specifying the type of postcracking behavior. Possible values are
            STRAIN, DISPLACEMENT, and GFI. The default value is STRAIN.

        Returns
        -------
        BrittleCracking
            A BrittleCracking object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the BrittleCracking object."""
        ...



from typing import Optional, Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..ConstrainedSketchBase import ConstrainedSketchBase
from ..ConstrainedSketchGeometry.ConstrainedSketchGeometry import (
    ConstrainedSketchGeometry,
)
from ..ConstrainedSketchVertex.ConstrainedSketchVertex import ConstrainedSketchVertex
from ...UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class ConstrainedSketchDimensionModel(ConstrainedSketchBase):
    """A ConstrainedSketch object contains the entities that are used to create a sketch. The
    objects include ConstrainedSketchGeometry objects contained in the ConstrainedSketchGeometry Repository,
    such as Line, Arc, and Spline. ConstrainedSketchVertex, ConstrainedSketchDimension, ConstrainedSketchConstraint, and ConstrainedSketchParameter objects are
    contained in their respective repositories.

    .. note::
        This object can be accessed by::

            import sketch
            mdb.models[name].sketches[name]
    """

    @abaqus_method_doc
    def AngularDimension(
        self,
        line1: ConstrainedSketchGeometry,
        line2: ConstrainedSketchGeometry,
        textPoint: Sequence[float],
        value: Optional[float] = None,
        reference: Boolean = OFF,
    ):
        """This method constructs a ConstrainedSketchDimension object between two
        ConstrainedSketchGeometry objects, with the given angle between them.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].AngularDimension

        Parameters
        ----------
        line1
            A ConstrainedSketchGeometry object specifying the first line.
        line2
            A ConstrainedSketchGeometry object specifying the second line.
        textPoint
            A pair of Floats specifying the location of the dimension text.
        value
            A Float specifying the angle between the two lines.
        reference
            A Boolean specifying whether the created dimension enforces the above value or if it
            simply measures the angle between two lines.

        Returns
        -------
        dimension
            A ConstrainedSketchDimension object
        """
        ...

    @abaqus_method_doc
    def HorizontalDimension(
        self,
        vertex1: ConstrainedSketchVertex,
        vertex2: ConstrainedSketchVertex,
        textPoint: Sequence[float],
        value: Optional[float] = None,
        reference: Boolean = OFF,
    ):
        """This method constructs a ConstrainedSketchDimension object between two vertices. A
        horizontal dimension indicates the horizontal distance along the **X** axis between two
        vertices.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].HorizontalDimension

        Parameters
        ----------
        vertex1
            A ConstrainedSketchVertex object specifying the first endpoint.
        vertex2
            A ConstrainedSketchVertex object specifying the second endpoint.
        textPoint
            A pair of Floats specifying the location of the dimension text.
        value
            A Float distance between the two vertices.
        reference
            A Boolean specifying whether the created dimension enforces the above value or if it
            simply measures the distance between the two vertices.

        Returns
        -------
        dimension
            A ConstrainedSketchDimension object
        """
        ...

    @abaqus_method_doc
    def ObliqueDimension(
        self,
        vertex1: ConstrainedSketchVertex,
        vertex2: ConstrainedSketchVertex,
        textPoint: Sequence[float],
        value: Optional[float] = None,
        reference: Boolean = OFF,
    ):
        """This method constructs a ConstrainedSketchDimension object between two vertices. An
        oblique dimension indicates the distance between two vertices.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].ObliqueDimension

        Parameters
        ----------
        vertex1
            A ConstrainedSketchVertex object specifying the first endpoint.
        vertex2
            A ConstrainedSketchVertex object specifying the second endpoint.
        textPoint
            A pair of Floats specifying the location of the dimension text.
        value
            A Float specifying the distance between the two ConstrainedSketchVertex objects.
        reference
            A Boolean specifying whether the created dimension enforces the above value or if it
            simply measures the distance between the two vertices.

        Returns
        -------
        dimension
            A ConstrainedSketchDimension object
        """
        ...

    @abaqus_method_doc
    def RadialDimension(
        self,
        curve: ConstrainedSketchGeometry,
        textPoint: Sequence[float],
        value: Optional[float] = None,
        reference: Boolean = OFF,
        majorRadius: Optional[float] = None,
        minorRadius: Optional[float] = None,
    ):
        """This method constructs a ConstrainedSketchDimension object on a circular or elliptical
        arc. A radial dimension indicates the radius of an arc or circle or the major or minor
        radius of an ellipse.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].RadialDimension

        Parameters
        ----------
        curve
            A ConstrainedSketchGeometry object specifying the circular or elliptical arc.
        textPoint
            A pair of Floats specifying the location of the dimension text.
        value
            A Float specifying the radius of the arc, circle or ellipse.
        reference
            A Boolean specifying whether the created dimension enforces the above value or if it
            simply measures the angle between two lines.
        majorRadius
            A Float specifying the major Radius if **curve** is an ellipse. This is mutually exclusive
            with **value** and **minorRadius**.
        minorRadius
            A Float specifying the minor Radius if **curve** is an ellipse. This is mutually exclusive
            with **value** and **majorRadius**.

        Returns
        -------
        dimension
            A ConstrainedSketchDimension object
        """
        ...

    @abaqus_method_doc
    def VerticalDimension(
        self,
        vertex1: ConstrainedSketchVertex,
        vertex2: ConstrainedSketchVertex,
        textPoint: Sequence[float],
        value: Optional[float] = None,
        reference: Boolean = OFF,
    ):
        """This method constructs a ConstrainedSketchDimension between two vertices. A vertical
        dimension controls the vertical distance along the **Y** axis between two vertices.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].VerticalDimension

        Parameters
        ----------
        vertex1
            A ConstrainedSketchVertex object specifying the first endpoint.
        vertex2
            A ConstrainedSketchVertex object specifying the second endpoint.
        textPoint
            A pair of Floats specifying the location of the dimension text.
        value
            A Float specifying the angle between the two lines.
        reference
            A Boolean specifying whether the created dimension enforces the above value or if it
            simply measures the angle between two lines.

        Returns
        -------
        dimension
            A ConstrainedSketchDimension object
        """
        ...

    @abaqus_method_doc
    def DistanceDimension(
        self,
        entity1: ConstrainedSketchVertex,
        entity2: ConstrainedSketchVertex,
        textPoint: Sequence[float],
        value: Optional[float] = None,
        reference: Boolean = OFF,
    ):
        """This method constructs a ConstrainedSketchDimension object between two
        ConstrainedSketchGeometry, or aConstrainedSketchVertex and ConstrainedSketchGeometry
        object. A distance dimension specifies the shortest distance between two entities.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].DistanceDimension

        Parameters
        ----------
        entity1
            A ConstrainedSketchVertex object or ConstrainedSketchGeometry object.
        entity2
            A ConstrainedSketchVertex object or ConstrainedSketchGeometry object.
        textPoint
            A pair of Floats specifying the location of the dimension text.
        value
            A Float specifying the angle between the two lines.
        reference
            A Boolean specifying whether the created dimension enforces the above value or if it
            simply measures the angle between two lines.

        Returns
        -------
        dimension
            A ConstrainedSketchDimension object
        """
        ...



"""
Parameter Identification
========================

This is a simple example of parameter identification. The compression model is a simple model with a cube of 1x1x1 m.
The lower surface is fixed and the upper surface is loaded with a pressure of 100 Pa. The goal is to find the Young's
modulus of the material that gives a maximum displacement of -0.1 m.

.. figure:: /images/model.png
    :width: 50%
    :align: center

    A simple compression model.

Theoretically, the Young's modulus of the material can be calculated with the following equation:

.. math::

    E = \\frac{\\sigma}{\\epsilon} = \\frac{P}{u_{\max}/h} = \\frac{100}{0.1 / 1.0} = 1000

The output of this script is:

.. code-block:: none

    Search results:
         modulus   fitness
    0       10.0  9.774944
    1      100.0  0.887494
    2     1000.0  0.001251
    3    10000.0  0.090125
    4   100000.0  0.099013
    5  1000000.0  0.099901

    Best modulus=1000.0 with fitness=0.0012505635619163569
"""
import os
import pandas as pd
import numpy as np


def fitness(x: float, maxdisp_expected: float = -0.1):
    # Change the working directory to a new folder
    wd = os.path.join(os.path.dirname(__file__), f"Job-E={x}")
    os.makedirs(wd, exist_ok=True)
    os.chdir(wd)

    # Run the model and read the output, the additional argument can be read by the Abaqus/Python script
    os.system(f"python ../compression.py {x},0.2")

    # Read the output and calculate the fitness
    data = pd.read_csv("data.csv")
    maxdisp = data["U3"].iloc[-1]
    return abs(maxdisp - maxdisp_expected)


def grid_search(search_space: list[float], expected: float):
    fs = []
    for x in search_space:
        print(f"Running model for E={x}")
        fs.append(fitness(x, expected))
    argmin = np.argmin(fs)
    best = search_space[argmin]
    print("Search results:", pd.DataFrame({"modulus": search_space, "fitness": fs}), sep="\n")
    print(f"\nBest modulus={best} with fitness={fs[argmin]}")
    return best


if __name__ == "__main__":
    E = grid_search([1e1, 1e2, 1e3, 1e4, 1e5, 1e6], -0.1)



from abaqus.DisplayGroup.Leaf import Leaf  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromDatums import LeafFromDatums  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromDisplayGroup import LeafFromDisplayGroup  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromGeometry import LeafFromGeometry  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromInstance import LeafFromInstance  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromInstanceElementLabels import LeafFromInstanceElementLabels  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromInstanceNodeLabels import LeafFromInstanceNodeLabels  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromMeshElementLabels import LeafFromMeshElementLabels  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromMeshNodeLabels import LeafFromMeshNodeLabels  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromMeshSurfaceSets import LeafFromMeshSurfaceSets  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromPartElementLabels import LeafFromPartElementLabels  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromPartNodeLabels import LeafFromPartNodeLabels  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromReferencePoint import LeafFromReferencePoint  # noqa # pylint: disable=unused-import
from abaqus.DisplayGroup.LeafFromSets import LeafFromSets  # noqa # pylint: disable=unused-import



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .BoundaryCondition import BoundaryCondition
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import NOT_APPLICABLE, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class TypeBC(BoundaryCondition):
    """The TypeBC object stores the data for several types of predefined boundary conditions
    that are commonly used in stress/displacement analyses.
    The TypeBC object is derived from the BoundaryCondition object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].boundaryConditions[name]
    """

    #: A String specifying the boundary condition repository key.
    name: str = ""

    #: A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
    #: analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
    #: PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
    buckleCase: SymbolicConstant = NOT_APPLICABLE

    #: A SymbolicConstant specifying the category of the boundary condition. Possible values
    #: are MECHANICAL and THERMAL.
    category: Optional[SymbolicConstant] = None

    #: A Region object specifying the region to which the boundary condition is applied.
    region: Region = Region()

    #: None or a DatumCsys object specifying the local coordinate system of the boundary
    #: condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
    #: in the global coordinate system. The default value is None.
    localCsys: Optional[str] = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: Literal[
            C.STRESS_PERTURBATION, C.NOT_APPLICABLE, C.BUCKLING_MODES, C.BUCKLE, C.PERTURBATION_AND_BUCKLING
        ] = NOT_APPLICABLE,
        localCsys: Optional[str] = None,
    ) -> None:
        """This method creates an TypeBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].EncastreBC
                mdb.models[name].PinnedBC
                mdb.models[name].XsymmBC
                mdb.models[name].YsymmBC
                mdb.models[name].ZsymmBC
                mdb.models[name].XasymmBC
                mdb.models[name].YasymmBC
                mdb.models[name].ZasymmBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        """
        super().__init__()
        self.name = name
        self.buckleCase = buckleCase
        self.region = region
        self.localCsys = localCsys

    @staticmethod
    @abaqus_method_doc
    def EncastreBC(
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: SymbolicConstant = NOT_APPLICABLE,
        localCsys: Optional[str] = None,
    ) -> "TypeBC":
        """This method creates an encastre TypeBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].EncastreBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        return TypeBC(name, createStepName, region, buckleCase, localCsys)

    @staticmethod
    @abaqus_method_doc
    def PinnedBC(
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: SymbolicConstant = NOT_APPLICABLE,
        localCsys: Optional[str] = None,
    ) -> "TypeBC":
        """This method creates a pinned TypeBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].PinnedBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        return TypeBC(name, createStepName, region, buckleCase, localCsys)

    @staticmethod
    @abaqus_method_doc
    def XsymmBC(
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: SymbolicConstant = NOT_APPLICABLE,
        localCsys: Optional[str] = None,
    ) -> "TypeBC":
        """This method creates a TypeBC object that specifies symmetry about the **X** axis.

        .. note::
            This function can be accessed by::

                mdb.models[name].XsymmBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        return TypeBC(name, createStepName, region, buckleCase, localCsys)

    @staticmethod
    @abaqus_method_doc
    def YsymmBC(
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: SymbolicConstant = NOT_APPLICABLE,
        localCsys: Optional[str] = None,
    ) -> "TypeBC":
        """This method creates a TypeBC object that specifies symmetry about the **Y** axis.

        .. note::
            This function can be accessed by::

                mdb.models[name].YsymmBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        return TypeBC(name, createStepName, region, buckleCase, localCsys)

    @staticmethod
    @abaqus_method_doc
    def ZsymmBC(
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: SymbolicConstant = NOT_APPLICABLE,
        localCsys: Optional[str] = None,
    ) -> "TypeBC":
        """This method creates a TypeBC object that specifies symmetry about the **Z** axis.

        .. note::
            This function can be accessed by::

                mdb.models[name].ZsymmBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        return TypeBC(name, createStepName, region, buckleCase, localCsys)

    @staticmethod
    @abaqus_method_doc
    def XasymmBC(
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: SymbolicConstant = NOT_APPLICABLE,
        localCsys: Optional[str] = None,
    ) -> "TypeBC":
        """This method creates a TypeBC object that specifies antisymmetry about the **X** axis.

        .. note::
            This function can be accessed by::

                mdb.models[name].XasymmBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        return TypeBC(name, createStepName, region, buckleCase, localCsys)

    @staticmethod
    @abaqus_method_doc
    def YasymmBC(
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: SymbolicConstant = NOT_APPLICABLE,
        localCsys: Optional[str] = None,
    ) -> "TypeBC":
        """This method creates a TypeBC object that specifies antisymmetry about the **Y** axis.

        .. note::
            This function can be accessed by::

                mdb.models[name].YasymmBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        return TypeBC(name, createStepName, region, buckleCase, localCsys)

    @staticmethod
    @abaqus_method_doc
    def ZasymmBC(
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: SymbolicConstant = NOT_APPLICABLE,
        localCsys: Optional[str] = None,
    ) -> "TypeBC":
        """This method creates a TypeBC object that specifies antisymmetry about the **Z** axis.

        .. note::
            This function can be accessed by::

                mdb.models[name].ZasymmBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        return TypeBC(name, createStepName, region, buckleCase, localCsys)

    @abaqus_method_doc
    def setValues(
        self,
        region: Region,
        typeName: Optional[
            Literal[C.YASYMM, C.ENCASTRE, C.ZASYMM, C.ZSYMM, C.YSYMM, C.XSYMM, C.PINNED, C.XASYMM]
        ] = None,
        buckleCase: Literal[
            C.STRESS_PERTURBATION, C.NOT_APPLICABLE, C.BUCKLING_MODES, C.BUCKLE, C.PERTURBATION_AND_BUCKLING
        ] = NOT_APPLICABLE,
        localCsys: Optional[str] = None,
    ):
        """This method modifies the data for an existing TypeBC object in the step where it is
        created.

        Parameters
        ----------
        region
            A Region object specifying the region to which the boundary condition is applied.
        typeName
            A SymbolicConstant specifying the predefined boundary condition type. Possible values
            are XSYMM, YSYMM, ZSYMM, XASYMM, YASYMM, ZASYMM, PINNED, and ENCASTRE.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(
        self,
        stepName: str,
        typeName: Optional[
            Literal[C.YASYMM, C.ENCASTRE, C.ZASYMM, C.ZSYMM, C.YSYMM, C.XSYMM, C.PINNED, C.XASYMM]
        ] = None,
    ):
        """This method always returns a value error for a TypeBC; it is inherited from the
        BoundaryCondition object.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the boundary condition is modified.
        typeName
            A SymbolicConstant specifying the predefined boundary condition type. Possible values
            are XSYMM, YSYMM, ZSYMM, XASYMM, YASYMM, ZASYMM, PINNED, and ENCASTRE.

        Raises
        ------
        Value Error
            A Symmetry/Antisymmetry/Encastre BC cannot be edited in a propagated step.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from .LoadState import LoadState
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class BodyForceState(LoadState):
    """The BodyForceState object stores the propagating data of a body force in a step. One
    instance of this object is created internally by the BodyForce object for each step. The
    instance is also deleted internally by the BodyForce object.
    The BodyForceState object has no constructor or methods.
    The BodyForceState object is derived from the LoadState object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].loadStates[name]

        The corresponding analysis keywords are:

        - DLOAD
    """

    #: A Float or a Complex specifying the body force component in the 1-direction.
    comp1: Optional[float] = None

    #: A Float or a Complex specifying the body force component in the 2-direction.
    comp2: Optional[float] = None

    #: A Float or a Complex specifying the body force component in the 3-direction.
    comp3: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the body force component in the
    #: 1-direction. Possible values are UNSET, SET, UNCHANGED, and MODIFIED.
    comp1State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the body force component in the
    #: 2-direction. Possible values are UNSET, SET, UNCHANGED, and MODIFIED.
    comp2State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the body force component in the
    #: 3-direction. Possible values are UNSET, SET, UNCHANGED, and MODIFIED.
    comp3State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the LoadState object. Possible
    #: values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    amplitude: str = ""



from importlib import metadata

from docutils import nodes
from sphinx.application import Sphinx
from sphinx.util.docutils import SphinxRole


class VersionRole(SphinxRole):
    """Role for inserting the version of a package. For example:

    .. code-block:: rst

        :version:`numpy`
        :version-major:`numpy`
        :version-minor:`numpy`
        :version-patch:`numpy`
        :version:`prefix text |numpy| suffix text`
    """
    type = 'version'

    def run(self):
        prefix, suffix, package = '', '', ''
        elements = self.text.split('|', 3)
        if len(elements) == 1:
            package = elements[0]
        elif len(elements) == 2:
            prefix, package = elements
        elif len(elements) == 3:
            prefix, package, suffix = elements
        else:
            raise ValueError('Invalid version role syntax')

        major, minor, patch = metadata.version(package).split('.')[:3]
        version = (major if self.type == 'major' else
                   minor if self.type == 'minor' else
                   patch if self.type == 'patch' else
                   f'{major}.{minor}.{patch}')
        text = f'{prefix}{version}{suffix}'
        return [nodes.Text(text, text)], []


class MajorVersionRole(VersionRole):
    type = 'major'


class MinorVersionRole(VersionRole):
    type = 'minor'


class PatchVersionRole(VersionRole):
    type = 'patch'


def setup(app: Sphinx):
    app.add_role('version', VersionRole())
    app.add_role('version-major', MajorVersionRole())
    app.add_role('version-minor', MinorVersionRole())
    app.add_role('version-patch', PatchVersionRole())
    return {'version': '0.0.1', 'parallel_read_safe': True}



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .....UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class CyclicHardening:
    r"""The CyclicHardening object defines the evolution of the elastic domain for the nonlinear
    isotropic/kinematic hardening model.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].Plastic.cyclicHardening
            import odbMaterial
            session.odbs[name].materials[name].Plastic.cyclicHardening

        The table data for this object are:

        - Equivalent stress.
        - :math:`Q_{\infty}` (only if **parameters** = ON).
        - Hardening parameter (only if **parameters** = ON).
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - CYCLIC HARDENING
    """

    @abaqus_method_doc
    def __init__(
        self,
        table: tuple,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        parameters: Boolean = OFF,
    ):
        """This method creates a CyclicHardening object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].Plastic.CyclicHardening
                session.odbs[name].materials[name].Plastic.CyclicHardening

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        parameters
            A Boolean specifying whether material parameters are to be input directly. The default
            value is OFF.

        Returns
        -------
        CyclicHardening
            A CyclicHardening object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the CyclicHardening object."""
        ...



"""
Parameter Identification: Model Script
======================================

Script to build the compression model for the parameter identification
"""

# run with: python compression.py 1000,0.2

from abaqus import *
from abaqusConstants import *
from caeModules import *
from driverUtils import *

import numpy as np

def run(modulus, poisson = 0.2):

    executeOnCaeStartup()

    # Model
    model = mdb.models['Model-1']

    # Part
    sketch = model.ConstrainedSketch(name='sketch', sheetSize=1.0)
    sketch.rectangle((0, 0), (1, 1))
    part = model.Part(name='part', dimensionality=THREE_D, type=DEFORMABLE_BODY)
    part.BaseSolidExtrude(sketch=sketch, depth=1)

    # Create sets
    part.Set(name='set-all', cells=part.cells.findAt(coordinates=((0.5, 0.5, 0.5),)))
    part.Set(name='set-bottom', faces=part.faces.findAt(coordinates=((0.5, 0.5, 0.0),)))
    part.Set(name='set-top', faces=part.faces.findAt(coordinates=((0.5, 0.5, 1.0),)))
    part.Surface(name='surface-top',
                 side1Faces=part.faces.findAt(coordinates=((0.5, 0.5, 1.0),)))

    # Assembly
    model.rootAssembly.DatumCsysByDefault(CARTESIAN)
    model.rootAssembly.Instance(name='instance', part=part, dependent=ON)

    # Material
    material = model.Material(name='material')
    material.Elastic(table=((modulus, poisson),))
    material.Density(table=((2500,),))

    # Section
    model.HomogeneousSolidSection(name='section', material='material', thickness=None)
    part.SectionAssignment(region=part.sets['set-all'], sectionName='section')

    # Step
    step = model.StaticStep(name='Step-1', previous='Initial', description='',
                            timePeriod=1.0, timeIncrementationMethod=AUTOMATIC,
                            maxNumInc=100, initialInc=0.01, minInc=0.001, maxInc=0.1)

    # Output request
    field = model.FieldOutputRequest('F-Output-1', createStepName='Step-1',
                                     variables=('S', 'E', 'U'))

    # Boundary condition
    bottom_instance = model.rootAssembly.instances['instance'].sets['set-bottom']
    bc = model.DisplacementBC(name='BC-1', createStepName='Initial',
                              region=bottom_instance, u1=SET, u2=SET, u3=SET, ur1=SET, ur2=SET, ur3=SET)

    # Load
    top_instance = model.rootAssembly.instances['instance'].surfaces['surface-top']
    pressure = model.Pressure('pressure', createStepName='Step-1', region=top_instance,
                              magnitude=100)

    # Mesh
    elem1 = mesh.ElemType(elemCode=C3D8R)
    elem2 = mesh.ElemType(elemCode=C3D6)
    elem3 = mesh.ElemType(elemCode=C3D4)
    part.setElementType(regions=(part.cells,), elemTypes=(elem1, elem2, elem3))
    part.seedPart(size=0.1)
    part.generateMesh()

    # Job
    job = mdb.Job(name='Job-1', model='Model-1')
    job.writeInput()

    # Submit the job
    job.submit()
    job.waitForCompletion()

    mdb.saveAs('compression.cae')

    # Open output database
    odb = session.openOdb('Job-1.odb')

    # Show the output database in viewport
    session.viewports['Viewport: 1'].setValues(displayedObject=odb)

    # Extract output data
    dataList = session.xyDataListFromField(odb=odb, outputPosition=NODAL,
                                           variable=(('U', NODAL, ((COMPONENT, 'U3'),)),),
                                           nodeSets=('INSTANCE.SET-TOP', ))

    data = np.array(dataList[0])
    np.savetxt('data.csv', data, header='time,U3', delimiter=',', comments='')


if __name__ == '__main__':
    E, mu = sys.argv[-1].split(',')
    run(float(E), float(mu))



from typing import Union, Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .Arrow import Arrow
from .Text import Text
from ..Odb.UserDataBase import UserDataBase
from ..UtilityAndView.abaqusConstants import (
    BOTTOM_LEFT,
    Boolean,
    FILLED_ARROW,
    JUSTIFY_LEFT,
    NONE,
    OFF,
    SOLID,
    TRANSPARENT,
    VERY_THIN,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class AnimationUserData(UserDataBase):
    @abaqus_method_doc
    def Arrow(
        self,
        name: str,
        startPoint: Sequence[float] = (0.0, 0.0),
        endPoint: Sequence[float] = (0.0, 0.0),
        startAnchor: Union[
            Literal[
                C.CENTER_RIGHT,
                C.TOP_CENTER,
                C.BOTTOM_RIGHT,
                C.BOTTOM_LEFT,
                C.CENTER,
                C.TOP_RIGHT,
                C.CENTER_LEFT,
                C.TOP_LEFT,
                C.BOTTOM_CENTER,
            ],
            float,
        ] = BOTTOM_LEFT,
        endAnchor: Union[
            Literal[
                C.CENTER_RIGHT,
                C.TOP_CENTER,
                C.BOTTOM_RIGHT,
                C.BOTTOM_LEFT,
                C.CENTER,
                C.TOP_RIGHT,
                C.CENTER_LEFT,
                C.TOP_LEFT,
                C.BOTTOM_CENTER,
            ],
            float,
        ] = BOTTOM_LEFT,
        startHeadStyle: Literal[
            C.FILLED_DIAMOND,
            C.FILLED_SQUARE,
            C.HOLLOW_CIRCLE,
            C.HOLLOW_SQUARE,
            C.FILLED_ARROW,
            C.HOLLOW_DIAMOND,
            C.ARROW,
            C.NONE,
            C.FILLED_CIRCLE,
        ] = NONE,
        endHeadStyle: Literal[
            C.FILLED_DIAMOND,
            C.FILLED_SQUARE,
            C.HOLLOW_CIRCLE,
            C.HOLLOW_SQUARE,
            C.FILLED_ARROW,
            C.HOLLOW_DIAMOND,
            C.ARROW,
            C.NONE,
            C.FILLED_CIRCLE,
        ] = FILLED_ARROW,
        startGap: float = 0.0,
        endGap: float = 0.0,
        color: str = "White",
        lineStyle: Literal[C.DASHED, C.SOLID, C.DOT_DASH, C.DOTTED] = SOLID,
        lineThickness: Literal[C.THIN, C.THICK, C.VERY_THIN, C.MEDIUM] = VERY_THIN,
    ) -> Arrow:
        """This method creates an Arrow object.

        .. note::
            This function can be accessed by::

                mdb.Arrow
                session.odbs[name].userData.Arrow

        Parameters
        ----------
        name
            A String specifying the annotation repository key.
        startPoint
            A pair of Floats specifying the start point **X**  and **Y** offsets in millimeters from
            **startAnchor**. The default value is (0, 0).
        endPoint
            A pair of Floats specifying the end point **X**  and **Y** offsets in millimeters from
            **endAnchor**. The default value is (0, 0).
        startAnchor
            A SymbolicConstant or a sequence of Floats specifying a point. A sequence of two Floats
            specifies the **X**  and **Y** coordinates as percentages of the viewport width and height.
            A sequence of three Floats specifies the **X**, **Y**, and **Z** coordinates of a point in
            the model coordinate system. A SymbolicConstant indicates a relative position. Possible
            values are:

            - BOTTOM_LEFT,,
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT

            The default value is BOTTOM_LEFT.
        endAnchor
            A SymbolicConstant or a sequence of Floats specifying a point. A sequence of two Floats
            specifies the **X**  and **Y** coordinates as percentages of the viewport width and height.
            A Sequence of three Floats specifies the **X**, **Y**, and **Z** coordinates of a point in
            the model coordinate system. A SymbolicConstant indicates a relative position. Possible
            values are:

            - BOTTOM_LEFT
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT

            The default value is BOTTOM_LEFT.
        startHeadStyle
            A SymbolicConstant specifying the style of the start head. Possible values are:

            - ARROW
            - FILLED_ARROW
            - HOLLOW_CIRCLE
            - FILLED_CIRCLE
            - HOLLOW_DIAMOND
            - FILLED_DIAMOND
            - HOLLOW_SQUARE
            - FILLED_SQUARE
            - NONE

            The default value is NONE.
        endHeadStyle
            A SymbolicConstant specifying the style of the end head. Possible values are:

            - ARROW
            - FILLED_ARROW
            - HOLLOW_CIRCLE
            - FILLED_CIRCLE
            - HOLLOW_DIAMOND
            - FILLED_DIAMOND
            - HOLLOW_SQUARE
            - FILLED_SQUARE
            - NONE

            The default value is FILLED_ARROW.
        startGap
            A Float specifying the distance in millimeters between the arrow start point and the
            arrow start head. The default value is 0.0.
        endGap
            A Float specifying the distance in millimeters between the arrow end point and the arrow
            end head. The default value is 0.0.
        color
            A String specifying the color of the arrow. Possible string values are any valid color.
            The default value is "White".
        lineStyle
            A SymbolicConstant specifying the line style of the arrow. Possible values are SOLID,
            DASHED, DOTTED, and DOT_DASH. The default value is SOLID.
        lineThickness
            A SymbolicConstant specifying the line thickness of the arrow. Possible values are
            VERY_THIN, THIN, MEDIUM, and THICK. The default value is VERY_THIN.

        Returns
        -------
        Arrow
            An Arrow object.
        """
        self.annotations[name] = arrow = Arrow(
            name,
            startPoint,
            endPoint,
            startAnchor,
            endAnchor,
            startHeadStyle,
            endHeadStyle,
            startGap,
            endGap,
            color,
            lineStyle,
            lineThickness,
        )
        return arrow

    @abaqus_method_doc
    def Text(
        self,
        name: str,
        text: str = "",
        offset: Sequence[float] = (),
        anchor: Union[
            Literal[
                C.CENTER_RIGHT,
                C.TOP_CENTER,
                C.BOTTOM_RIGHT,
                C.BOTTOM_LEFT,
                C.CENTER,
                C.TOP_RIGHT,
                C.CENTER_LEFT,
                C.TOP_LEFT,
                C.BOTTOM_CENTER,
            ],
            float,
        ] = BOTTOM_LEFT,
        referencePoint: Union[
            Literal[
                C.CENTER_RIGHT,
                C.TOP_CENTER,
                C.BOTTOM_RIGHT,
                C.BOTTOM_LEFT,
                C.CENTER,
                C.TOP_RIGHT,
                C.CENTER_LEFT,
                C.TOP_LEFT,
                C.BOTTOM_CENTER,
            ],
            float,
        ] = BOTTOM_LEFT,
        rotationAngle: float = 0,
        color: str = "White",
        font: str = "-*-verdana-medium-r-normal--120-*",
        backgroundStyle: Literal[C.OTHER, C.MATCH, C.TRANSPARENT] = TRANSPARENT,
        backgroundColor: str = "",
        box: Boolean = OFF,
        justification: Literal[C.JUSTIFY_CENTER, C.JUSTIFY_LEFT, C.JUSTIFY_RIGHT] = JUSTIFY_LEFT,
    ) -> Text:
        """This method creates a Text object.

        .. note::
            This function can be accessed by::

                mdb.Text
                session.odbs[name].userData.Text

        Parameters
        ----------
        name
            A String specifying the annotation repository key.
        text
            A String specifying the text of the Text object. The default value is an empty string.
        offset
            A pair of Floats specifying the **X**  and **Y** offsets in millimeters of the Text object
            from **anchor**. The default value is (0, 0).
        anchor
            A SymbolicConstant or a sequence of Floats specifying a point. A sequence of two Floats
            specifies the **X**  and **Y** coordinates as percentages of the viewport width and height.
            A Sequence of three Floats specifies the **X**, **Y**, and **Z** coordinates of a point in
            the model coordinate system. A SymbolicConstant specifies a relative position. Possible
            values are:

            - BOTTOM_LEFT
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT

            The default value is BOTTOM_LEFT.
        referencePoint
            A SymbolicConstant or a sequence of Floats specifying a point. The sequence of two
            Floats specifies the **X**  and **Y** coordinates of the reference point of the Text
            annotation given as percentages of its width and height. The SymbolicConstant indicates
            a relative position. Possible values are:

            - BOTTOM_LEFT
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT

            The default value is BOTTOM_LEFT.
        rotationAngle
            A Float specifying the amount of rotation in degrees about **referencePoint**. The default
            value is 0.0.
        color
            A String specifying the color of the Text object. Possible string values are any valid
            color. The default value is "White".
        font
            A String specifying the font of the Text object. Possible string values are any valid
            font specification. The default value is "-*-verdana-medium-r-normal--120-*".
        backgroundStyle
            A SymbolicConstant specifying the Text object background style. Possible values are
            MATCH, TRANSPARENT, and OTHER. The default value is TRANSPARENT.
        backgroundColor
            A String specifying the color of the Text object background. Possible string values are
            any valid color. The default value matches the viewport background.
        box
            A Boolean specifying whether the box around the text is shown. The default value is OFF.
        justification
            A SymbolicConstant specifying the Text object justification for multiline text. Possible
            values are JUSTIFY_LEFT, JUSTIFY_CENTER, and JUSTIFY_RIGHT. The default value is
            JUSTIFY_LEFT.

        Returns
        -------
        Text
            A Text object.
        """
        self.annotations[name] = tex = Text(
            name,
            text,
            offset,
            anchor,
            referencePoint,
            rotationAngle,
            color,
            font,
            backgroundStyle,
            backgroundColor,
            box,
            justification,
        )
        return tex



from typing import Optional, Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ..UtilityAndView.abaqusConstants import BEST_FIT, Boolean, CENTER, FIT_TO_VIEWPORT, OFF, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .._OptionsBase import _CopyOptionsBase


@abaqus_class_doc
class ImageOptions(_CopyOptionsBase):
    """The ImageOptions object stores settings that control how an image is rendered in a
    particular viewport. ImageOptions objects are accessed from the image options associated
    with a particular viewport.
    The ImageOptions object has no constructor; Abaqus creates the **imageOptions** member for
    a viewport when the viewport is created using the values in the **imageOptions** member of
    the current viewport.

    .. note::
        This object can be accessed by::

            session.viewports[name].imageOptions
    """

    #: A String specifying the name of the image. A list of valid image names is in the
    #: **images** repository in the **session** object.
    imageName: str = ""

    #: A Boolean specifying whether an image should be displayed in the viewport background.
    #: The default value is OFF.
    showImage: Boolean = OFF

    #: A SymbolicConstant specifying which positioning method is used to determine how the
    #: image will be scaled and positioned in the viewport. Possible values are:
    #:
    #: - FIT_TO_VIEWPORT, specifying a display mode where the image is scaled to fit in the
    #:   viewport using the specified **fitMethod**.
    #: - AUTO_ALIGN, specifying a display mode where the
    #:   image is scaled as specified by **xScale** and **yScale** and then positioned in the
    #:   viewport using the specified **alignment**.
    #: - MANUAL, specifying a display mode where the
    #:   image is scaled as specified by **xScale** and **yScale** and then positioned in the
    #:   viewport using the specified **origin**.
    #:
    #: The default value is FIT_TO_VIEWPORT.
    positionMethod: SymbolicConstant = FIT_TO_VIEWPORT

    #: A SymbolicConstant specifying which type of fit is performed to scale and position the
    #: image in the viewport. Possible values are:
    #:
    #: - BEST_FIT, specifying a mode where the image is scaled to completely fit within the
    #:   viewport.
    #: - FIT_WIDTH, specifying a mode where the image width is scaled to match the
    #:   viewport width.
    #: - FIT_HEIGHT, specifying a mode where the image height is scaled to match
    #:   the viewport height.
    #:
    #: The default value is BEST_FIT.
    fitMethod: SymbolicConstant = BEST_FIT

    #: A SymbolicConstant specifying the relative position of the image in the viewport when
    #: **positionMethod** =AUTO_ALIGN. Possible values are:
    #:
    #: - BOTTOM_LEFT
    #: - BOTTOM_CENTER
    #: - BOTTOM_RIGHT
    #: - CENTER_LEFT
    #: - CENTER
    #: - CENTER_RIGHT
    #: - TOP_LEFT
    #: - TOP_CENTER
    #: - TOP_RIGHT
    #:
    #: The default value is CENTER.
    alignment: SymbolicConstant = CENTER

    #: A Float specifying the scale applied to the image width. The **xScale** argument is
    #: ignored when **positionMethod** =FIT_TO_VIEWPORT. The default value is 1.0.When **xScale**
    #: is negative, the image is mirrored about its y-axis but its position is not affected.
    xScale: float = 1.0

    #: A Float specifying the scale applied to the image height. The **yScale** argument is
    #: ignored when **positionMethod** =FIT_TO_VIEWPORT. The default value is 1.0.When **yScale**
    #: is negative, the image is mirrored about its x-axis but its position is not affected.
    yScale: float = 1.0

    #: A pair of Floats specifying the **X**  and **Y** offsets in millimeters from the lower-left
    #: corner of the viewport. The **origin** argument is ignored unless **positionMethod**
    #: =MANUAL. The default value is (0, 0).
    origin: Sequence[float] = (0.0, 0.0)

    #: A Float specifying the translucency factor to use when displaying the image. Possible
    #: values are 0.0 ≤ **translucency** ≤ 1.0 with 0.0 being invisible and 1.0 being opaque.
    #: The default value is 1.0.
    translucency: float = 1.0

    @abaqus_method_doc
    def setValues(
        self,
        imageName: str = "",
        showImage: Boolean = OFF,
        positionMethod: Literal[C.FIT_TO_VIEWPORT, C.MANUAL, C.AUTO_ALIGN] = FIT_TO_VIEWPORT,
        fitMethod: Literal[C.FIT_TO_VIEWPORT, C.FIT_WIDTH, C.BEST_FIT, C.FIT_HEIGHT] = BEST_FIT,
        alignment: Literal[
            C.CENTER_RIGHT,
            C.TOP_CENTER,
            C.BOTTOM_RIGHT,
            C.BOTTOM_LEFT,
            C.CENTER,
            C.AUTO_ALIGN,
            C.TOP_RIGHT,
            C.CENTER_LEFT,
            C.TOP_LEFT,
            C.BOTTOM_CENTER,
        ] = CENTER,
        xScale: float = 1.0,
        yScale: float = 1.0,
        origin: Sequence[float] = (0.0, 0.0),
        translucency: float = 1.0,
        options: Optional["ImageOptions"] = None,
    ):
        """This method modifies the ImageOptions object.

        Parameters
        ----------
        imageName
            A String specifying the name of the image. A list of valid image names is in the
            **images** repository in the **session** object.
        showImage
            A Boolean specifying whether an image should be displayed in the viewport background.
            The default value is OFF.
        positionMethod
            A SymbolicConstant specifying which positioning method is used to determine how the
            image will be scaled and positioned in the viewport. Possible values are:

            - FIT_TO_VIEWPORT, specifying a display mode where the image is scaled to fit in the
              viewport using the specified **fitMethod**.
            - AUTO_ALIGN, specifying a display mode where the
              image is scaled as specified by **xScale** and **yScale** and then positioned in the
              viewport using the specified **alignment**.
            - MANUAL, specifying a display mode where the
              image is scaled as specified by **xScale** and **yScale** and then positioned in the
              viewport using the specified **origin**.

            The default value is FIT_TO_VIEWPORT.
        fitMethod
            A SymbolicConstant specifying which type of fit is performed to scale and position the
            image in the viewport when **positionMethod** =FIT_TO_VIEWPORT. Possible values are:

            - BEST_FIT, specifying a mode where the image is scaled to completely fit within the
              viewport.
            - FIT_WIDTH, specifying a mode where the image width is scaled to match the
              viewport width.
            - FIT_HEIGHT, specifying a mode where the image height is scaled to match
              the viewport height.

            The default value is BEST_FIT.
        alignment
            A SymbolicConstant specifying the relative position of the image in the viewport when
            **positionMethod** =AUTO_ALIGN. Possible values are:

            - BOTTOM_LEFT
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT

            The default value is CENTER.
        xScale
            A Float specifying the scale applied to the image width. The **xScale** argument is
            ignored when **positionMethod** =FIT_TO_VIEWPORT. The default value is 1.0.When **xScale**
            is negative, the image is mirrored about its y-axis but its position is not affected.
        yScale
            A Float specifying the scale applied to the image height. The **yScale** argument is
            ignored when **positionMethod** =FIT_TO_VIEWPORT. The default value is 1.0.When **yScale**
            is negative, the image is mirrored about its x-axis but its position is not affected.
        origin
            A pair of Floats specifying the **X**  and **Y** offsets in millimeters from the lower-left
            corner of the viewport. The **origin** argument is ignored unless **positionMethod**
            =MANUAL. The default value is (0, 0).
        translucency
            A Float specifying the translucency factor to use when displaying the image. Possible
            values are 0.0 ≤ **translucency** ≤ 1.0 with 0.0 being invisible and 1.0 being opaque.
            The default value is 1.0.
        options
            None or an ImageOptions object specifying the object from which values are to be copied.
            If other arguments are also supplied to setValues, they will override the values in the
            **options** member. The default value is None.

        Raises
        ------
        RangeError
        """
        super().setValues(
            options=options,
            imageName=imageName,
            showImage=showImage,
            positionMethod=positionMethod,
            fitMethod=fitMethod,
            alignment=alignment,
            xScale=xScale,
            yScale=yScale,
            origin=origin,
            translucency=translucency,
        )



from __future__ import annotations

from typing import Union, Tuple, Sequence, Dict, List, overload

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .Face import Face
from ..UtilityAndView.abaqusConstants import Boolean


@abaqus_class_doc
class FaceArray(List[Face]):
    """The FaceArray is a sequence of Face objects. If the part is modified, then FaceArray
    must be updated for that part.

    .. note::
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].allInternalSets[name].faces
            mdb.models[name].parts[name].allInternalSurfaces[name].faces
            mdb.models[name].parts[name].allSets[name].faces
            mdb.models[name].parts[name].allSurfaces[name].faces
            mdb.models[name].parts[name].faces
            mdb.models[name].parts[name].sets[name].faces
            mdb.models[name].parts[name].surfaces[name].faces
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].faces
            mdb.models[name].rootAssembly.allInstances[name].sets[name].faces
            mdb.models[name].rootAssembly.allInstances[name].surfaces[name].faces
            mdb.models[name].rootAssembly.allInternalSets[name].faces
            mdb.models[name].rootAssembly.allInternalSurfaces[name].faces
            mdb.models[name].rootAssembly.allSets[name].faces
            mdb.models[name].rootAssembly.allSurfaces[name].faces
            mdb.models[name].rootAssembly.instances[name].faces
            mdb.models[name].rootAssembly.instances[name].sets[name].faces
            mdb.models[name].rootAssembly.instances[name].surfaces[name].faces
            mdb.models[name].rootAssembly.modelInstances[i].sets[name].faces
            mdb.models[name].rootAssembly.modelInstances[i].surfaces[name].faces
            mdb.models[name].rootAssembly.sets[name].faces
            mdb.models[name].rootAssembly.surfaces[name].faces
    """

    @abaqus_method_doc
    def __init__(self, faces: List[Face]) -> None:
        """This method creates a FaceArray object.

        .. note::
            This function can be accessed by::

                part.FaceArray

        Parameters
        ----------
        faces
            A list of Face objects.

        Returns
        -------
        FaceArray
            A FaceArray object.

        """
        ...

    @overload
    @abaqus_method_doc
    def findAt(
        self,
        coordinates: Tuple[float, float, float],
        printWarning: Boolean = True,
    ) -> Face:
        ...

    @overload
    @abaqus_method_doc
    def findAt(
        self,
        coordinates: Tuple[
            Tuple[float, float, float],
        ],
        printWarning: Boolean = True,
    ) -> List[Face]:
        ...

    @overload
    @abaqus_method_doc
    def findAt(
        self,
        *coordinates: Tuple[
            Tuple[float, float, float],
        ],
        printWarning: Boolean = True,
    ) -> List[Face]:
        ...

    @abaqus_method_doc
    def findAt(self, *args, **kwargs) -> Union[Face, List[Face]]:
        """This method returns the object or objects in the FaceArray located at the given
        coordinates.
        findAt initially uses the ACIS tolerance of 1E-6. As a result, findAt returns any face
        that is at the arbitrary point specified or at a distance of less than 1E-6 from the
        arbitrary point. If nothing is found, findAt uses the tolerance for imprecise geometry
        (applicable only for imprecise geometric entities). The arbitrary point must not be
        shared by a second face. If two faces intersect or coincide at the arbitrary point,
        findAt chooses the first face that it encounters, and you should not rely on the return
        value being consistent.
        findAt will always try to find objects among all the faces in the part or assembly
        instance and will not restrict itself to a subset even if the FaceArray represents such
        subset.

        Parameters
        ----------
        coordinates
            A sequence of Floats specifying the **X**, **Y**, and **Z** coordinates of the object to
            find. `findAt` returns either a Face object or a sequence of Face objects based on the type
            of input.

            * If **coordinates** is a sequence of Floats, findAt returns the Face object at that point.

            * If you omit the **coordinates** keyword argument, findAt accepts as arguments a
              sequence of pairs of sequences describing each face's coordinate and normal, and findAt
              returns a sequence of Face objects at the given locations. If you omit the **coordinates**
              keyword argument, you must also omit the **normal** argument::

                faces = f.findAt(((-16.438578, -41.835673, -24.19804), ),
                                 ((25.210364, -35.689868, 1.860314), ),
                                 ((26.727683, -38.207055, 4.164759), ))
        normal
            A sequence of Floats specifying the **X**, **Y**, and **Z** components of a vector
            indicating the face normal.
        printWarning
            A Boolean specifying whether a message is to be printed to the CLI if no entity is found
            at the specified location. The default value is True.

        Returns
        -------
        Face
            A Face object.

        """
        first_arg = kwargs.get("coordinates", args[0] if args else ((),))
        return Face() if isinstance(first_arg[0], float) else [Face()]

    @abaqus_method_doc
    def getSequenceFromMask(self, mask: str):
        """This method returns the object or objects in the FaceArray identified using the
        specified **mask**. This command is generated when the JournalOptions are set to
        COMPRESSEDINDEX. When a large number of objects are involved, this method is highly
        efficient.

        Parameters
        ----------
        mask
            A String specifying the object or objects.

        Returns
        -------
        Face
            A Face object or a sequence of Face objects.

        """
        ...

    @abaqus_method_doc
    def getMask(self) -> str:
        """This method returns a string specifying the object or objects.

        Returns
        -------
        str
            A String specifying the object or objects.

        """
        ...

    @abaqus_method_doc
    def getByBoundingBox(
        self,
        xMin: float = ...,
        yMin: float = ...,
        zMin: float = ...,
        xMax: float = ...,
        yMax: float = ...,
        zMax: float = ...,
    ) -> FaceArray:
        """This method returns an array of face objects that lie within the specified bounding box.

        Parameters
        ----------
        xMin
            A float specifying the minimum **X** boundary of the bounding box.
        yMin
            A float specifying the minimum **Y** boundary of the bounding box.
        zMin
            A float specifying the minimum **Z** boundary of the bounding box.
        xMax
            A float specifying the maximum **X** boundary of the bounding box.
        yMax
            A float specifying the maximum **Y** boundary of the bounding box.
        zMax
            A float specifying the maximum **Z** boundary of the bounding box.

        Returns
        -------
        FaceArray
            A FaceArray object, which is a sequence of Face objects.

        """
        ...

    @abaqus_method_doc
    def getByBoundingCylinder(self, center1: tuple, center2: tuple, radius: str) -> FaceArray:
        """This method returns an array of face objects that lie within the specified bounding
        cylinder.

        Parameters
        ----------
        center1
            A tuple of the **X**, **Y**, and **Z** coordinates of the center of the first end of the
            cylinder.
        center2
            A tuple of the **X**, **Y**, and **Z** coordinates of the center of the second end of the
            cylinder.
        radius
            A float specifying the radius of the cylinder.

        Returns
        -------
        FaceArray
            A FaceArray object, which is a sequence of Face objects.

        """
        ...

    @abaqus_method_doc
    def getByBoundingSphere(self, center: tuple, radius: str) -> FaceArray:
        """This method returns an array of face objects that lie within the specified bounding
        sphere.

        Parameters
        ----------
        center
            A tuple of the **X**, **Y**, and **Z** coordinates of the center of the sphere.
        radius
            A float specifying the radius of the sphere.

        Returns
        -------
        FaceArray
            A FaceArray object, which is a sequence of Face objects.

        """
        ...

    @abaqus_method_doc
    def getBoundingBox(self) -> Dict[str, Sequence[float]]:
        """This method returns a dictionary of two tuples representing minimum and maximum boundary
        values of the bounding box of the minimum size containing the face sequence.

        Returns
        -------
        Dict[str, Sequence[float]]
            A Dictionary object with the following items:

            - **low**: a tuple of three floats representing the minimum **X** -, **Y** -, and **Z**  -boundary
              values of the bounding box.
            - **high**: a tuple of three floats representing the maximum **X** -, **Y** -, and **Z**  -boundary
              values of the bounding box.
        """
        ...

    @abaqus_method_doc
    def getClosest(self, coordinates: tuple, searchTolerance: str = "") -> Dict:
        """This method returns an object or objects in the FaceArray closest to the given set of
        points, where the given points need not lie on the faces in the FaceArray.

        Parameters
        ----------
        coordinates
            A sequence of a sequence of floats, where each sequence of floats describes the **X**,
            **Y**, and **Z** coordinates of a point::

                >>> r=f.getClosest(coordinates=((20.0, 20.0, 10.0), (-1.0, -15.0, 15), ))
                >>> r.keys()
                [0, 1]
                >>> r[0]
                (mdb.models['Model-1'].parts['Part-1'].faces[0], (15.7090625762939, 20.0, 10.0))
        searchTolerance
            A double specifying the distance within which the closest object must lie. The default
            value is half of the parent part/instance size.

        Returns
        -------
        dict
            This method returns a dictionary object. The key to the dictionary object is the
            position of the input point in the tuple specified in the **coordinates** starting at
            index 0. If a closest face could be found then the value is a sequence consisting of two
            objects. The first object in the sequence is a Face that is close to the input point
            referred to by the key. The second object in the sequence is a sequence of floats that
            specifies the **X**, **Y**, and **Z**  location of the closest point on the Face to the given
            point. See program listing above.

        Raises
        ------
        Error
            The mask results in an empty sequence, An exception occurs if the resulting sequence is empty.
        """
        ...



from __future__ import annotations

from typing import Optional, Sequence, Tuple

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ..UtilityAndView.abaqusConstants import ABSOLUTE, MODEL, OFF, Boolean
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class View:
    """The Session and Viewport View objects store view settings for custom (both predefined
    and user-defined) views. The paradigm used to define a view is based on a camera
    analogy. Similar to taking a photograph with a camera, features such as camera position,
    view direction, orientation, depth of field, and projection are specified to transform
    three-dimensional views to the screen.
    The Layer View objects store a transformation matrix used to position the contents of
    the Layer within a viewport.

    .. note::
        This object can be accessed by::

            session.viewports[name].layers[name].view
            session.viewports[name].view
            session.views[name]
    """

    #: A Float specifying the width in viewport millimeters of the bounding rectangle around
    #: the viewport contents. This value does not include annotations or symbols and it is not
    #: clipped to the size of the viewport window.
    displayedObjectScreenWidth: Optional[float] = None

    #: A Float specifying the height in viewport millimeters of the bounding rectangle around
    #: the viewport contents. This value does not include annotations or symbols and it is not
    #: clipped to the size of the viewport window.
    displayedObjectScreenHeight: Optional[float] = None

    #: A tuple of Floats specifying a transformation matrix used to position the contents of
    #: the Layer within a viewport.
    layerTransform: Optional[float] = None

    #: A String specifying the name of the view (also used as the repository key). Possible
    #: values are 'Front', 'Back', 'Top', 'Bottom', 'Left', 'Right', 'Iso', 'User-1', 'User-2',
    #: 'User-3', and 'User-4'. The object member associated with this argument is a
    #: SymbolicConstant. Possible values of the **name** member are: FRONT, BACK, TOP, BOTTOM,
    #: LEFT, RIGHT, ISO, USER1, USER2, USER3, and USER4.
    name: str

    #: A Float specifying the distance from the camera to the near clipping plane. Possible
    #: values are **nearPlane** > 0.0.
    nearPlane: float

    #: A Float specifying the distance from the camera to the far clipping plane when
    #: **farPlaneMode** =SPECIFY. Possible values are **farPlane** > **nearPlane**.
    farPlane: float

    #: A Float specifying the width of the front clipping plane. Possible values are **width** >
    #: 0.0.
    width: float

    #: A Float specifying the height of the front clipping plane. Possible values are **height**
    #: > 0.0.
    height: float

    #: A SymbolicConstant specifying the projection mode. Possible values are PERSPECTIVE and
    #: PARALLEL.
    projection: Literal[C.PERSPECTIVE, C.PARALLEL]

    #: A sequence of three Floats specifying the camera position.
    cameraPosition: Tuple[float, float, float]

    #: A sequence of three Floats specifying the camera's up vector (the screen's positive
    #: **Y** axis). The initial value is (0, 0, 0).
    cameraUpVector: Tuple[float, float, float]

    #: A sequence of three Floats specifying the center of the scene.
    cameraTarget: Tuple[float, float, float]

    #: A Float specifying the amount to pan the model in the screen **X** direction as a fraction
    #: of the viewport width. A positive value pans the model to the right. A negative value
    #: pans the model to the left. The *viewOffsetX* and **viewOffsetY** arguments allow you to pan
    #: the view without changing the position of the camera or the target (*cameraPosition* and
    #: **cameraTarget** arguments to the View method). The resulting change in the view allows
    #: you to pan a perspective display without producing an apparent rotation of the model.
    viewOffsetX: float

    #: A Float specifying the amount to pan the model in the screen **Y** direction as a fraction
    #: of the viewport height. A positive value pans the model upward. A negative value pans
    #: the model downward.
    viewOffsetY: float

    #: A Boolean specifying whether the view is auto-fit when applied.
    autoFit: Boolean

    #: A Boolean specifying whether or not the camera is in movie mode. The default value is
    #: OFF.
    movieMode: Boolean = OFF

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        nearPlane: float,
        farPlane: float,
        width: float,
        height: float,
        projection: Literal[C.PERSPECTIVE, C.PARALLEL],
        cameraPosition: Tuple[float, float, float],
        cameraUpVector: Tuple[float, float, float],
        cameraTarget: Tuple[float, float, float],
        viewOffsetX: float,
        viewOffsetY: float,
        autoFit: Boolean,
        movieMode: Boolean = OFF,
    ):
        """This method creates a View object.
        Note:All dimensions and coordinates are specified in the model coordinate system.
        Note:This method cannot be used to create a View for a Layer object.

        .. note::
            This function can be accessed by::

                session.View

        Parameters
        ----------
        name
            A String specifying the name of the view (also used as the repository key). Possible
            values are 'Front', 'Back', 'Top', 'Bottom', 'Left', 'Right', 'Iso', 'User-1', 'User-2',
            'User-3', and 'User-4'. The object member associated with this argument is a
            SymbolicConstant. Possible values of the **name** member are:FRONT, BACK, TOP, BOTTOM,
            LEFT, RIGHT, ISO, USER1, USER2, USER3, and USER4.
        nearPlane
            A Float specifying the distance from the camera to the near clipping plane. Possible
            values are **nearPlane** > 0.0.
        farPlane
            A Float specifying the distance from the camera to the far clipping plane when
            **farPlaneMode** =SPECIFY. Possible values are **farPlane** > **nearPlane**.
        width
            A Float specifying the width of the front clipping plane. Possible values are **width** >
            0.0.
        height
            A Float specifying the height of the front clipping plane. Possible values are **height**
            > 0.0.
        projection
            A SymbolicConstant specifying the projection mode. Possible values are PERSPECTIVE and
            PARALLEL.
        cameraPosition
            A sequence of three Floats specifying the camera position.
        cameraUpVector
            A sequence of three Floats specifying the camera's up vector (the screen's positive
            **Y** axis). The initial value is (0, 0, 0).
        cameraTarget
            A sequence of three Floats specifying the center of the scene.
        viewOffsetX
            A Float specifying the amount to pan the model in the screen **X** direction as a fraction
            of the viewport width. A positive value pans the model to the right. A negative value
            pans the model to the left. The *viewOffsetX* and **viewOffsetY** arguments allow you to pan
            the view without changing the position of the camera or the target (*cameraPosition* and
            **cameraTarget** arguments to the View method). The resulting change in the view allows
            you to pan a perspective display without producing an apparent rotation of the model.
        viewOffsetY
            A Float specifying the amount to pan the model in the screen **Y** direction as a fraction
            of the viewport height. A positive value pans the model upward. A negative value pans
            the model downward.
        autoFit
            A Boolean specifying whether the view is auto-fit when applied.
        movieMode
            A Boolean specifying whether or not the camera is in movie mode. The default value is
            OFF.

        Returns
        -------
        View
            A View object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def fitView(self, drawImmediately: Boolean = False):
        """This method scales the displayable object (such as a part, the assembly, or an **X - Y**
        plot) to fit the viewport.

        Parameters
        ----------
        drawImmediately
            A Boolean specifying the viewport should refresh immediately after the command is
            processed. This is typically only used when writing a script and it is desirable to show
            intermediate results before the script completes. The default value is False.
        """
        ...

    @abaqus_method_doc
    def next(self, drawImmediately: Boolean = False):
        """This method restores the view in the viewport to the next view setting in the list.
        (There is a list of eight views stored for each viewport.) If there is no next view, no
        action is taken.
        Note:This method is not available for a Layer View.

        Parameters
        ----------
        drawImmediately
            A Boolean specifying the viewport should refresh immediately after the command is
            processed. This is typically only used when writing a script and it is desirable to show
            intermediate results before the script completes. The default value is False.
        """
        ...

    @abaqus_method_doc
    def pan(
        self,
        xFraction: float = 0,
        yFraction: float = 0,
        asMovie: Boolean = OFF,
        drawImmediately: Boolean = False,
    ):
        """This method pans the view in the viewport using absolute, not relative, mode.

        Parameters
        ----------
        xFraction
            A Float specifying the amount to pan the model in the screen **X** direction as a fraction
            of the viewport width. A positive value pans the model to the right. A negative value
            pans the model to the left. The default value is 0.0.
        yFraction
            A Float specifying the amount to pan the model in the screen **Y** direction as a fraction
            of the viewport height. A positive value pans the model upward. A negative value pans
            the model downward. The default value is 0.0.
        asMovie
            A Boolean specifying the alternate mode of the pan view manipulation should be used. The
            default value is OFF. This argument is ignored for a Layer View.
        drawImmediately
            A Boolean specifying the viewport should refresh immediately after the command is
            processed. This argument is typically used only when writing a script and it is
            desirable to show intermediate results before the script completes. The default value is
            False.
        """
        ...

    @abaqus_method_doc
    def previous(self, drawImmediately: Boolean = False):
        """This method restores the view in the viewport to the previous view setting in the list.
        (There is a list of eight views stored for each viewport.) If there is no previous view,
        no action is taken.
        Note:This method is not available for a Layer View.

        Parameters
        ----------
        drawImmediately
            A Boolean specifying the viewport should refresh immediately after the command is
            processed. This argument is typically used only when writing a script and it is
            desirable to show intermediate results before the script completes. The default value is
            False.
        """
        ...

    @abaqus_method_doc
    def rotate(
        self,
        xAngle: float = 0,
        yAngle: float = 0,
        zAngle: float = 0,
        mode: Literal[C.TOTAL, C.SCREEN, C.MODEL] = MODEL,
        asMovie: Boolean = OFF,
        drawImmediately: Boolean = False,
    ):
        """This method rotates the view in the viewport. If a center of rotation has been
        previously specified and **asMovie** is OFF then this method will honor that rotation
        center.

        Parameters
        ----------
        xAngle
            A Float specifying the degrees to rotate about the **X** axis. The default value is 0.0.
        yAngle
            A Float specifying the degrees to rotate about the **Y** axis. The default value is 0.0.
        zAngle
            A Float specifying the degrees to rotate about the*Z*-axis. The default value is 0.0.
        mode
            A SymbolicConstant specifying the rotation mode. Possible values are:

            * TOTAL : Set the view to (0, 0, 1), then rotate about the screen's axes (an absolute rotation).
            * SCREEN : Rotate incrementally about the screen's axes (a relative rotation).
            * MODEL : Rotate incrementally about the model's axes (a relative rotation).

            The default value is MODEL.
        asMovie
            A Boolean specifying the alternate mode of the rotate view manipulation should be used.
            The default value is OFF.
        drawImmediately
            A Boolean specifying the viewport should refresh immediately after the command is
            processed. This argument is typically used only when writing a script and it is
            desirable to show intermediate results before the script completes. The default value is
            False.
        """
        ...

    @abaqus_method_doc
    def setLayerTransform(
        self,
        layerTransform: tuple = (),
        options: Optional["View"] = None,
        drawImmediately: Boolean = False,
    ):
        """This method modifies the transformation used to position a Layer.
        Note:This method is not available for Session and Viewport Views.

        Parameters
        ----------
        layerTransform
            A sequence of 16 Floats specifying the transformation matrix.
        options
            A View object from which the view settings are to be copied. If the **layerTransform**
            argument is also supplied to setLayerTransform, it will override the values in the View
            object specified by **view**.
        drawImmediately
            A Boolean specifying the viewport should refresh immediately after the command is
            processed. This argument is typically used only when writing a script and it is
            desirable to show intermediate results before the script completes. The default value is
            False.
        """
        ...

    @abaqus_method_doc
    def setProjection(self, projection: Literal[C.PARALLEL, C.PERSPECTIVE], drawImmediately: Boolean = False):
        """This method modifies the appearance of three-dimensional models in the viewport.
        Choosing PERSPECTIVE makes a model appear more realistic by decreasing the apparent size
        of features that are farther away from the viewing point.
        Note:This method is not available for a Layer View.

        Parameters
        ----------
        projection
            A SymbolicConstant specifying the projection mode. Possible values are PERSPECTIVE and
            PARALLEL.
        drawImmediately
            A Boolean specifying the viewport should refresh immediately after the command is
            processed. This argument is typically used only when writing a script and it is
            desirable to show intermediate results before the script completes. The default value is
            False.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setRotationCenter(self, rotationCenter: tuple):
        """This method sets the center of rotation to the specified location.

        Parameters
        ----------
        rotationCenter
            A sequence of a String and an Int specifying a part instance name and a node label or a
            sequence of 3 Floats specifying a point.

        Raises
        ------
        TypeError
            rotationCenter cannot be set using a part instance and node label unless the
            displayed object is an ODB.
        """
        ...

    @abaqus_method_doc
    def setValues(
        self,
        options: View = ...,
        drawImmediately: Boolean = False,
        fieldOfViewAngle: float = ...,
        farPlaneMode: Literal[C.AUTOCOMPUTE, C.SPECIFY] = ...,
        nearPlane: float = ...,
        farPlane: float = ...,
        width: float = ...,
        height: float = ...,
        projection: Literal[C.PERSPECTIVE, C.PARALLEL] = ...,
        cameraPosition: Tuple[float, float, float] = ...,
        cameraUpVector: Tuple[float, float, float] = ...,
        cameraTarget: Tuple[float, float, float] = ...,
        viewOffsetX: float = ...,
        viewOffsetY: float = ...,
        movieMode: Boolean = OFF,
    ) -> None:
        """This method modifies the View object.

        .. note::
            This method is not available for a Layer View.

        The optional arguments to `setValues` are the same as the arguments to the
        View method, except for the `name`
        and `autoFit` arguments. In addition, `setValues` has the following optional
        arguments:

        Parameters
        ----------
        options
            A View object from which the view settings are to be copied. If other
            arguments are also supplied to `setValues`, they will override the values
            in the View object specified by `view`.

        drawImmediately
            A Boolean specifying the viewport should refresh immediately after the
            command is processed. This argument is typically used only when writing a
            script and it is desirable to show intermediate results before the script
            completes. The default value is False.

        fieldOfViewAngle
            A Float specifying the viewing angle of the camera. Possible values are
            0.0 < filedOfViewAngle < 180.0.

        farPlaneMode
            A SymbolicConstant specifying how the distance from the camera to the far
            clipping plane is set. Possible values are AUTOCOMPUTE and SPECIFY.

        Returns
        -------
        None.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setViewpoint(
        self,
        viewVector: tuple,
        cameraUpVector: tuple = (),
        drawImmediately: Boolean = False,
    ):
        """This method sets the camera's position in the viewport.
        Note:This method is not available for a Layer View.

        Parameters
        ----------
        viewVector
            A sequence of three Floats specifying the viewing vector (from the camera to the origin
            of the model).
        cameraUpVector
            A sequence of three Floats specifying the camera's up vector (the screen's positive
            **Y** axis). The initial value is (0, 0, 0).
        drawImmediately
            A Boolean specifying the viewport should refresh immediately after the command is
            processed. This argument is typically used only when writing a script and it is
            desirable to show intermediate results before the script completes. The default value is
            False.
        """
        ...

    @abaqus_method_doc
    def zoom(
        self,
        zoomFactor: float,
        mode: Literal[C.ABSOLUTE, C.RELATIVE] = ABSOLUTE,
        asMovie: Boolean = OFF,
        drawImmediately: Boolean = False,
    ):
        """This method magnifies the view in the viewport.

        Parameters
        ----------
        zoomFactor
            A Float specifying the amount to zoom. Possible values are 0.000001 ≤ **zoomFactor** ≤
            1000000. A **zoomFactor** less than one reduces the image. A **zoomFactor** greater than one
            enlarges the image.
        mode
            A SymbolicConstant specifying the way the zoom is executed. Possible values are:

            * ABSOLUTE : Execute fitView, then zoom.
            * RELATIVE : Zoom from the current camera settings.

            The default value is ABSOLUTE.
        asMovie
            A Boolean specifying the alternate mode of the zoom view manipulation should be used.
            The default value is OFF. This argument is ignored for a Layer View.
        drawImmediately
            A Boolean specifying the viewport should refresh immediately after the command is
            processed. This argument is typically used only when writing a script and it is
            desirable to show intermediate results before the script completes. The default value is
            False.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def zoomRectangle(
        self,
        point1: Sequence[float],
        point2: Sequence[float],
        drawImmediately: Boolean = False,
    ):
        """This method fills the viewport with the graphics located within the given rectangle.

        Parameters
        ----------
        point1
            A pair of Floats specifying the*X*- and **Y** coordinates of one corner of the rectangle
            in fractions of the viewport width and height.
        point2
            A pair of Floats specifying the*X*- and **Y** coordinates of the other corner of the
            rectangle in fractions of the viewport width and height.
        drawImmediately
            A Boolean specifying the viewport should refresh immediately after the command is
            processed. This argument is typically used only when writing a script and it is
            desirable to show intermediate results before the script completes. The default value is
            False.
        """
        ...



from sphinx.application import Sphinx
from sphinx.ext.autodoc import ClassDocumenter, ALL


class AbqpyClassDocumenter(ClassDocumenter):
    """Toggles the class members section"""

    def document_members(self, all_members: bool = False) -> None:
        # find out all the members that are documented
        want_all = all_members or self.options.inherited_members or self.options.members is ALL
        _, members = self.get_object_members(want_all)
        if members:
            sourcename = self.get_sourcename()
            self.add_line('', sourcename)
            self.add_line('.. rubric:: Member Details:', sourcename)
            self.add_line('', sourcename)
            self.add_line('.. toggle::', sourcename)
            self.add_line('', sourcename)
            self.add_line('    .. placeholder for empty members', sourcename)
            self.add_line('', sourcename)
            self.indent += ' ' * 4
        super().document_members(all_members)


def setup(app: Sphinx):
    app.setup_extension('sphinx.ext.autodoc')  # Require autodoc extension
    app.setup_extension('sphinx_togglebutton')  # Require sphinx-togglebutton extension
    app.add_autodocumenter(AbqpyClassDocumenter, override=True)
    return {
        'version': '0.0.1',
        'parallel_read_safe': True,
        'parallel_write_safe': True,
    }



from typing import Union, Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .AdjustPoints import AdjustPoints
from .Coupling import Coupling
from .DisplayBody import DisplayBody
from .EmbeddedRegion import EmbeddedRegion
from .Equation import Equation
from .MultipointConstraint import MultipointConstraint
from .RigidBody import RigidBody
from .ShellSolidCoupling import ShellSolidCoupling
from .Tie import Tie
from ..Assembly.PartInstance import PartInstance
from ..BasicGeometry.ModelDotArray import ModelDotArray
from ..Model.ModelBase import ModelBase
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import (
    BOTH,
    Boolean,
    COMPUTED,
    DEFAULT,
    DOF_MODE_MPC,
    OFF,
    ON,
    SOLVER_DEFAULT,
    UNIFORM,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class ConstraintModel(ModelBase):
    """Abaqus creates a Model object named `Model-1` when a session is started.

    .. note::
        This object can be accessed by::

            mdb.models[name]
    """

    @abaqus_method_doc
    def AdjustPoints(self, name: str, surface: Region, controlPoints: Region) -> AdjustPoints:
        """This method creates an AdjustPoints object.

        .. note::
            This function can be accessed by::

                mdb.models[name].AdjustPoints

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        surface
            A Region object specifying the surface to which the **controlPoints** are adjusted.
        controlPoints
            A Region object specifying the constraint control points.

        Returns
        -------
        AdjustPoints
            An AdjustPoints object.
        """
        self.constraints[name] = constraint = AdjustPoints(name, surface, controlPoints)
        return constraint

    @abaqus_method_doc
    def Coupling(
        self,
        name: str,
        surface: Region,
        controlPoint: Region,
        influenceRadius: Union[Literal[C.WHOLE_SURFACE], float],
        couplingType: Literal[C.STRUCTURAL, C.DISTRIBUTING, C.KINEMATIC],
        adjust: Boolean = OFF,
        localCsys: Optional[str] = None,
        u1: Boolean = ON,
        u2: Boolean = ON,
        u3: Boolean = ON,
        ur1: Boolean = ON,
        ur2: Boolean = ON,
        ur3: Boolean = ON,
        weightingMethod: Literal[C.QUADRATIC, C.DISTRIBUTING, C.UNIFORM, C.LINEAR, C.CUBIC] = UNIFORM,
    ) -> Coupling:
        """This method creates a Coupling object.

        .. note::
            This function can be accessed by::

                mdb.models[name].Coupling

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        surface
            A Region object specifying the surface on which the coupling nodes are located.
        controlPoint
            A Region object specifying the constraint control point.
        influenceRadius
            The SymbolicConstant WHOLE_SURFACE or a Float specifying the influence radius.
        couplingType
            A SymbolicConstant specifying the coupling constraint type. Possible values are
            KINEMATIC, DISTRIBUTING, and STRUCTURAL.
        adjust
            A Boolean specifying if the control point will be adjusted (moved) to the surface. The
            point will be adjusted in the direction normal to the specified surface. The default
            value is OFF.
        localCsys
            None or a DatumCsys object specifying the initial orientation of the local coordinate
            system for the coupling's degrees of freedom. If **localCsys** = None, the coupling is
            defined in the global coordinate system. The default value is None.
        u1
            A Boolean specifying if the displacement component in the 1-direction is constrained to
            the reference node for a kinematic coupling constraint. The default value is ON.The **u1**
            argument applies only when **couplingType** = KINEMATIC.
        u2
            A Boolean specifying if the displacement component in the 2-direction is constrained to
            the reference node for a kinematic coupling constraint. The default value is ON.The **u2**
            argument applies only when **couplingType** = KINEMATIC.
        u3
            A Boolean specifying if the displacement component in the 3-direction is constrained to
            the reference node for a kinematic coupling constraint. The default value is ON.The **u3**
            argument applies only when **couplingType** = KINEMATIC.
        ur1
            A Boolean specifying if the rotational displacement component about the 1-direction is
            constrained to the reference node for a kinematic coupling constraint. The default value
            is ON.The **ur1** argument applies only when **couplingType** = KINEMATIC.
        ur2
            A Boolean specifying if the rotational displacement component about the 2-direction is
            constrained to the reference node for a kinematic coupling constraint. The default value
            is ON.The **ur2** argument applies only when **couplingType** = KINEMATIC.
        ur3
            A Boolean specifying if the rotational displacement component about the 3-direction is
            constrained to the reference node for a kinematic coupling constraint. The default value
            is ON.The **ur3** argument applies only when **couplingType** = KINEMATIC.
        weightingMethod
            A SymbolicConstant specifying an optional weighting method used for calculating the
            distributing weight factors. Possible values are UNIFORM, LINEAR, QUADRATIC, and CUBIC.
            The default value is UNIFORM.The **weightingMethod** argument applies only when
            **couplingType** = DISTRIBUTING.

        Returns
        -------
        Coupling
            A Coupling object.
        """
        self.constraints[name] = constraint = Coupling(
            name,
            surface,
            controlPoint,
            influenceRadius,
            couplingType,
            adjust,
            localCsys,
            u1,
            u2,
            u3,
            ur1,
            ur2,
            ur3,
            weightingMethod,
        )
        return constraint

    @abaqus_method_doc
    def DisplayBody(self, name: str, instance: PartInstance, controlPoints: ModelDotArray) -> DisplayBody:
        """This method creates a DisplayBody object.

        .. note::
            This function can be accessed by::

                mdb.models[name].DisplayBody

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        instance
            A PartInstance object specifying the part instance that is to be used for display only.
        controlPoints
            A ModelDotArray object specifying the motion of the PartInstance. The control points may
            be ConstrainedSketchVertex, ReferencePoint, or MeshNode objects. Their motion will control the motion of
            the PartInstance. If this argument is set to an empty sequence, the PartInstance will
            remain fixed in space during the analysis. The sequence can have either one object or
            three objects.

        Returns
        -------
        DisplayBody
            A DisplayBody object.
        """
        self.constraints[name] = constraint = DisplayBody(name, instance, controlPoints)
        return constraint

    @abaqus_method_doc
    def EmbeddedRegion(
        self,
        name: str,
        embeddedRegion: Region,
        hostRegion: Region,
        weightFactorTolerance: Optional[float] = None,
        toleranceMethod: Literal[C.FRACTIONAL, C.BOTH, C.ABSOLUTE] = BOTH,
        absoluteTolerance: float = 0,
        fractionalTolerance: float = 0,
    ) -> EmbeddedRegion:
        """This method creates a EmbeddedRegion object.

        .. note::
            This function can be accessed by::

                mdb.models[name].EmbeddedRegion

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        embeddedRegion
            A Region object specifying the body region to be embedded.
        hostRegion
            A Region object specifying the host region. A value of None indicates that the host
            region is the whole model.
        weightFactorTolerance
            A Float specifying a small value below which the weighting factors will be zeroed out.
            The default value is 10⁻⁶.
        toleranceMethod
            A SymbolicConstant specifying the method used to determine the embedded element
            tolerance. Possible values are ABSOLUTE, FRACTIONAL, and BOTH. The default value is
            BOTH.
        absoluteTolerance
            A Float specifying the absolute value by which a node on the embedded region may lie
            outside the host region. If **absoluteTolerance** = 0.0, the **fractionalTolerance** value
            will be used. The default value is 0.0.This argument applies only when
            **toleranceMethod** = ABSOLUTE or BOTH.
        fractionalTolerance
            A Float specifying the fractional value by which a node on the embedded region may lie
            outside the host region. The fractional value is based on the average element size
            within the host region. The default value is 0.05.If both tolerance arguments are
            specified, the smaller value will be used.This argument applies only when
            **toleranceMethod** = FRACTIONAL or BOTH.

        Returns
        -------
        EmbeddedRegion
            An EmbeddedRegion object.
        """
        self.constraints[name] = constraint = EmbeddedRegion(
            name,
            embeddedRegion,
            hostRegion,
            weightFactorTolerance,
            toleranceMethod,
            absoluteTolerance,
            fractionalTolerance,
        )
        return constraint

    @abaqus_method_doc
    def Equation(self, name: str, terms: tuple) -> Equation:
        """This method creates an Equation object.

        .. note::
            This function can be accessed by::

                mdb.models[name].Equation

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        terms
            A sequence of (Float, String, Int, Int) sequences specifying a coefficient, Set name,
            degree of freedom, and coordinate system ID. The coordinate system ID is optional.

        Returns
        -------
        Equation
            An Equation object.

        Raises
        ------
        Exception
            If **terms** does not contain more than one entry, Equation must have two or more terms.
        """
        self.constraints[name] = constraint = Equation(name, terms)
        return constraint

    @abaqus_method_doc
    def MultipointConstraint(
        self,
        name: str,
        surface: Region,
        controlPoint: Region,
        mpcType: Literal[C.USER_MPC, C.BEAM_MPC, C.ELBOW_MPC, C.TIE_MPC, C.MPC, C.PIN_MPC, C.LINK_MPC],
        csys: Optional[str] = None,
        userType: int = 0,
        userMode: Literal[C.DOF_MODE_MPC, C.USER_MPC, C.NODE_MODE_MPC] = DOF_MODE_MPC,
    ) -> MultipointConstraint:
        """This method creates a MultipointConstraint object.

        .. note::
            This function can be accessed by::

                mdb.models[name].MultipointConstraint

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        surface
            A Region object specifying the surface on which the MultipointConstraint nodes are
            located.
        controlPoint
            A Region object specifying the constraint control point.
        mpcType
            A SymbolicConstant specifying the MPC type of the constraint. Possible values are
            BEAM_MPC, ELBOW_MPC, PIN_MPC, LINK_MPC, TIE_MPC, and USER_MPC.
        csys
            None or a DatumCsys object specifying the initial orientation of the local coordinate
            system for the MultipointConstraint's degrees of freedom. If **localCsys** = None, the
            MultipointConstraint is defined in the global coordinate system. The default value is
            None.
        userType
            An Int specifying to differentiate between different constraint types in a user-defined
            MultipointConstraint. The default value is 0.The **userType** argument applies only when
            **mpcType** = USER_MPC.
        userMode
            A SymbolicConstant specifying the mode of the constraint when it is user-defined.
            Possible values are DOF_MODE_MPC and NODE_MODE_MPC. The default value is
            DOF_MODE_MPC.The **userMode** argument applies only when **mpcType** = USER_MPC.

        Returns
        -------
        MultipointConstraint
            A MultipointConstraint object.
        """
        self.constraints[name] = constraint = MultipointConstraint(
            name, surface, controlPoint, mpcType, csys, userType, userMode
        )
        return constraint

    @abaqus_method_doc
    def RigidBody(
        self,
        name: str,
        refPointRegion: Region,
        bodyRegion: Optional[str] = None,
        tieRegion: Optional[str] = None,
        pinRegion: Optional[str] = None,
        surfaceRegion: Optional[str] = None,
        refPointAtCOM: Boolean = OFF,
        isothermal: Boolean = OFF,
    ) -> RigidBody:
        """This method creates a RigidBody object.

        .. note::
            This function can be accessed by::

                mdb.models[name].RigidBody

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        refPointRegion
            A Region object specifying the reference point.
        bodyRegion
            None or a Region object specifying the elements constrained to the movement of the
            reference point. The default value is None.
        tieRegion
            None or a Region object specifying the nodes tied to the movement of the reference
            point. The default value is None.
        pinRegion
            None or a Region object specifying the nodes pinned to the movement of the reference
            point. The default value is None.
        surfaceRegion
            None or a Region object specifying the analytic surface constrained to the movement of
            the reference point. The default value is None.
        refPointAtCOM
            A Boolean specifying whether the analysis product should recompute the reference point
            position to be at the center of mass. The default value is OFF.
        isothermal
            A Boolean specifying whether the temperature degree of freedom should be constrained.
            The default value is OFF.

        Returns
        -------
        RigidBody
            A RigidBody object.
        """
        self.constraints[name] = constraint = RigidBody(
            name,
            refPointRegion,
            bodyRegion,
            tieRegion,
            pinRegion,
            surfaceRegion,
            refPointAtCOM,
            isothermal,
        )
        return constraint

    @abaqus_method_doc
    def ShellSolidCoupling(
        self,
        name: str,
        shellEdge: Region,
        solidFace: Region,
        positionToleranceMethod: Literal[C.COMPUTED, C.SPECIFIED] = COMPUTED,
        positionTolerance: float = 0,
        influenceDistanceMethod: Literal[C.DEFAULT, C.SPECIFIED] = DEFAULT,
        influenceDistance: float = 0,
    ) -> ShellSolidCoupling:
        """This method creates a ShellSolidCoupling object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ShellSolidCoupling

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        shellEdge
            A Region object specifying the name of the shell edge surface.
        solidFace
            A Region object specifying the name of the solid surface.
        positionToleranceMethod
            A SymbolicConstant specifying the method used to determine the position tolerance.
            Possible values are COMPUTED and SPECIFIED. The default value is COMPUTED.
        positionTolerance
            A Float specifying the position tolerance. The default value is 0.0.The
            **positionTolerance** argument applies only when
            **positionToleranceMethod** = SPECIFIED. Note: Abaqus will not constrain nodes on the solid
            face region outside the position tolerance.
        influenceDistanceMethod
            A SymbolicConstant specifying the method used to determine the influence distance.
            Possible values are DEFAULT and SPECIFIED. The default value is DEFAULT.
        influenceDistance
            A Float specifying the influence distance. The **influenceDistance** argument applies only
            when **influenceDistanceMethod** = SPECIFIED. The default value is 0.0.

        Returns
        -------
        ShellSolidCoupling
            A ShellSolidCoupling object.
        """
        self.constraints[name] = constraint = ShellSolidCoupling(
            name,
            shellEdge,
            solidFace,
            positionToleranceMethod,
            positionTolerance,
            influenceDistanceMethod,
            influenceDistance,
        )
        return constraint

    @abaqus_method_doc
    def Tie(
        self,
        name: str,
        master: Region,
        slave: Region,
        adjust: Boolean = ON,
        positionToleranceMethod: Literal[C.COMPUTED, C.SPECIFIED] = COMPUTED,
        positionTolerance: float = 0,
        tieRotations: Boolean = ON,
        constraintRatioMethod: Literal[C.DEFAULT, C.SPECIFIED] = DEFAULT,
        constraintRatio: float = 0,
        constraintEnforcement: Literal[C.NODE_TO_SURFACE, C.SOLVER_DEFAULT, C.SURFACE_TO_SURFACE] = SOLVER_DEFAULT,
        thickness: Boolean = ON,
    ) -> Tie:
        """This method creates a Tie object.

        .. note::
            This function can be accessed by::

                mdb.models[name].Tie

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        master
            A Region object specifying the name of the master surface.
        slave
            A Region object specifying the name of the slave surface.
        adjust
            A Boolean specifying whether initial positions of tied slave nodes are adjusted to
            lie on the master surface. The default value is ON.
        positionToleranceMethod
            A SymbolicConstant specifying the method used to determine the position tolerance.
            Possible values are COMPUTED and SPECIFIED. The default value is COMPUTED.
        positionTolerance
            A Float specifying the position tolerance. The **positionTolerance** argument applies only
            when **positionToleranceMethod** = SPECIFIED. The default value is 0.0.
        tieRotations
            A Boolean specifying whether rotation degrees of freedom should be tied. The default
            value is ON.
        constraintRatioMethod
            A SymbolicConstant specifying the method used to determine the constraint ratio.
            Possible values are DEFAULT and SPECIFIED. The default value is DEFAULT.
        constraintRatio
            A Float specifying the fractional distance between the master reference surface and the
            slave node at which the translational constraint should act. The **constraintRatio**
            argument applies only when **constraintRatioMethod** = SPECIFIED. The default value is 0.0.
        constraintEnforcement
            A SymbolicConstant specifying the discretization method. Possible values are
            SOLVER_DEFAULT, NODE_TO_SURFACE, and SURFACE_TO_SURFACE. The default value is
            SOLVER_DEFAULT.
        thickness
            A Boolean specifying whether shell element thickness is considered. The default value is
            ON.

        Returns
        -------
        Tie
            A Tie object.
        """
        self.constraints[name] = constraint = Tie(
            name,
            master,
            slave,
            adjust,
            positionToleranceMethod,
            positionTolerance,
            tieRotations,
            constraintRatioMethod,
            constraintRatio,
            constraintEnforcement,
            thickness,
        )
        return constraint
